var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, ElementRef, EventEmitter, Input, Output, ViewEncapsulation } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import { GitHubLinkService } from './github-link.service';
/**
 * GitHub Link Area component. This shows a formatted link to GitHub with an
 * indicator of whether the GitHub issue is open or closed in an text area.
 * Works only for public remote repositories for now. See example for how to
 * use this.
 */
var GitHubLinkAreaComponent = /** @class */ (function () {
    function GitHubLinkAreaComponent(gitHubLinkService, elementRef, sanitizer) {
        this.gitHubLinkService = gitHubLinkService;
        this.elementRef = elementRef;
        this.sanitizer = sanitizer;
        this.onInputEvent = new EventEmitter();
    }
    GitHubLinkAreaComponent.prototype.ngOnChanges = function (changes) {
        if (changes && changes.content) {
            this.updateOnChanges();
        }
    };
    GitHubLinkAreaComponent.prototype.ngAfterViewChecked = function () {
        this.instrumentInputs();
    };
    /*
     * Because Angular testing is broken, we need to be able to call
     * this from a test. See reason here:
     * https://github.com/angular/angular/issues/9866
     */
    GitHubLinkAreaComponent.prototype.updateOnChanges = function () {
        this.updateLinkTexts();
        this.updateLinks();
    };
    /*
     * Instruments the input elements in the content. This is used to report
     * any interaction with inputs back to the parent using Outputs. This feature
     * is used in the checkbox feature downstream.
     */
    GitHubLinkAreaComponent.prototype.instrumentInputs = function () {
        var _this = this;
        // this uses a trick. The problem is that ngAfterViewChecked() is called
        // multiple times and causes EventListeners added to the inputs multiple
        // times. Using a dirty bit does not work as Angular is ansychronous and
        // updates can happen between setting the EventListener and reverting the
        // dirty bit.
        // keeping track of the elements with EventListeners is also not working
        // without a lot of extra work. So we do a trick and add a custom attribute
        // to the input element when we added the EventListener. The selector
        // matches only elements without that attribute, so we have a lightweight
        // way of making sure each input element exactly gets one EventListener
        // attached.
        var el = this.elementRef;
        if (el) {
            var inputElements = el.nativeElement.querySelectorAll('input:not([data-event-attached])');
            if (inputElements && inputElements.length > 0) {
                // we need to use a classic loop instead of forEach here
                // as forEach on NodeLists is not supported on every browser.
                // Example: Chrome works, but Protractor not.
                for (var i = 0; i < inputElements.length; ++i) {
                    inputElements[i].setAttribute('data-event-attached', 'true');
                    inputElements[i].addEventListener('change', function (ref) {
                        // we only support checkboxes for now, but the mechanism is generic.
                        // add new interactions here if needed.
                        if (ref.target && ref.target.getAttribute('type') === 'checkbox') {
                            var indexStr = ref.target.getAttribute('data-checkbox-index');
                            _this.onInputEvent.emit({
                                'type': 'checkbox',
                                // '+' converts the string to an int.
                                'extraData': {
                                    checkboxIndex: +indexStr,
                                    checked: ref.target.checked
                                }
                            });
                        }
                    }, false);
                }
            }
        }
    };
    GitHubLinkAreaComponent.prototype.wrapStringSafeValue = function (input) {
        if (typeof input === 'string') {
            return this.sanitizer.bypassSecurityTrustHtml(input);
        }
        else {
            return input;
        }
    };
    GitHubLinkAreaComponent.prototype.unwrapStringSafeValue = function (input) {
        if (typeof input === 'string') {
            return input;
        }
        else {
            return input['changingThisBreaksApplicationSecurity'];
        }
    };
    /*
     * Replaces the match (which should be the default icon) with an icon that
     * indicates the status contained in linkData.
     */
    GitHubLinkAreaComponent.prototype.replaceLink = function (linkData) {
        this.content = this.wrapStringSafeValue(this.unwrapStringSafeValue(this.content).split(linkData.match).join(
        // tslint:disable-next-line:max-line-length
        (linkData.state === 'open' ? '<span class="fa fa-clock-o gh-link-open" tooltip="Issue Open"></span>' : '') +
            // tslint:disable-next-line:max-line-length
            (linkData.state === 'closed' ? '<span class="fa fa-check gh-link-closed" tooltip="Issue Closed"></span>' : '') +
            ((linkData.state !== 'open' && linkData.state !== 'closed') ?
                // tslint:disable-next-line:max-line-length
                '<span class="fa pficon-warning-triangle-o gh-link-error" tooltip="Issue State Unknown"></span>' : '')));
    };
    /*
     * This does a preliminary synchonous replacement of the original html link with
     * a formatted version and an "unknown status" icon. This first step is needed to
     * not "flash" the original links in the field before updating the icons. In the
     * next step (in updateLinks()), the "unknown status" icon is replaced with the
     * actual status icon.
     */
    GitHubLinkAreaComponent.prototype.updateLinkTexts = function () {
        // the following regexp only matches a specific way of links, namely links with no
        // additional classes etc.; if a markdown compiler (or some other content source)
        // creates different links to GitHub, this regexp needs to be extended to match
        // those formats.
        var thisContent = this.unwrapStringSafeValue(this.content);
        var regexp = new RegExp('<a href="https:\/\/github.com\/([^\/]+)\/([^\/]+)\/issues\/([^"]+)[^<]*">([^<]+)<\/a>', 'gi');
        var result = regexp.exec(thisContent);
        while (result) {
            thisContent = thisContent.split(result[0])
                .join('<a class="gh-link" href="https://github.com/' +
                result[1] + '/' +
                result[2] + '/' +
                'issues/' + result[3] + '" rel="nofollow">' +
                '<span class="fa fa-github gh-link-system"></span><span class="gh-link-label"> ' +
                result[2] + ':' + result[3] + ' ' +
                '<span ' +
                'data-gh-org="' + result[1] + '" ' +
                'data-gh-repo="' + result[2] + '" ' +
                'data-gh-issue="' + result[3] + '" ' +
                'class="pficon pficon-warning-triangle-o gh-link-error"></span>' +
                '</a>');
            result = regexp.exec(thisContent);
        }
        this.content = this.wrapStringSafeValue(thisContent);
    };
    /*
     * This updates the icons asynchonously and only replaces the "unknown status" icon
     * with the actual icon matching the remote status. The data is retrieved using the
     * GitHubLinkService to use caching.
     */
    GitHubLinkAreaComponent.prototype.updateLinks = function () {
        var _this = this;
        var thisContent = this.unwrapStringSafeValue(this.content);
        var regexp = new RegExp(
        // tslint:disable-next-line:max-line-length
        '<span data-gh-org="([^"]+)" data-gh-repo="([^"]+)" data-gh-issue="([^"]+)" class="pficon pficon-warning-triangle-o gh-link-error"></span>', 'gi');
        var result = regexp.exec(thisContent);
        var _loop_1 = function () {
            var thisLinkData = {
                match: result[0],
                org: result[1],
                repo: result[2],
                issue: result[3],
                state: 'error'
            };
            this_1.gitHubLinkService.getIssue(thisLinkData)
                .subscribe(function (data) {
                thisLinkData.state = data['state'];
                _this.replaceLink(thisLinkData);
            });
            result = regexp.exec(thisContent);
        };
        var this_1 = this;
        while (result) {
            _loop_1();
        }
    };
    __decorate([
        Input('content'),
        __metadata("design:type", Object)
    ], GitHubLinkAreaComponent.prototype, "content", void 0);
    __decorate([
        Output('onInputEvent'),
        __metadata("design:type", Object)
    ], GitHubLinkAreaComponent.prototype, "onInputEvent", void 0);
    GitHubLinkAreaComponent = __decorate([
        Component({
            encapsulation: ViewEncapsulation.None,
            selector: 'github-link-area',
            styles: [require('./github-link-area.component.css').toString()],
            template: "<div [innerHTML]=\"content\"></div>"
        }),
        __metadata("design:paramtypes", [GitHubLinkService,
            ElementRef,
            DomSanitizer])
    ], GitHubLinkAreaComponent);
    return GitHubLinkAreaComponent;
}());
export { GitHubLinkAreaComponent };
//# sourceMappingURL=github-link-area.component.js.map