var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { ChangeDetectorRef, Component, ElementRef, EventEmitter, HostListener, Input, Output, ViewChild, ViewEncapsulation } from '@angular/core';
var MarkdownComponent = /** @class */ (function () {
    function MarkdownComponent(cdr, elementRef) {
        this.cdr = cdr;
        this.elementRef = elementRef;
        this.fieldName = 'Description';
        this.inpRenderedText = '';
        this.inpRawText = '';
        this.rendering = false;
        this.saving = false;
        this.placeholder = 'This is place holder';
        this.editAllow = true;
        this.renderedHeight = 300;
        this.allowEmptySave = true;
        this.onActiveEditor = new EventEmitter();
        this.onSaveClick = new EventEmitter();
        this.showPreview = new EventEmitter();
        this.onCloseClick = new EventEmitter();
        this.onClickOut = new EventEmitter();
        this.enableShowMore = false;
        // these need to be public for the tests accessing them.
        this.renderedText = '';
        this.rawText = '';
        this.fieldEmpty = true;
        this.previousRawText = '';
        this.isNoDataChanged = false;
        this.markdownViewExpanded = false;
        this.tabBarVisible = true;
        this.viewType = 'preview'; // markdown
        this.editorActive = false;
        this.showMore = false;
        this.inputsDisabled = false;
        this.previousRenderedText = '';
    }
    MarkdownComponent.prototype.onClick = function (target) {
        if (!target) {
            return;
        }
        var clickedInside = this.elementRef.nativeElement.contains(target);
        if (this.editorInput) {
            this.isNoDataChanged = this.editorInput.nativeElement.innerText.trim() === this.rawText;
        }
        if (!clickedInside) {
            this.onClickOut.emit(this.isNoDataChanged);
        }
    };
    MarkdownComponent.prototype.ngOnChanges = function (changes) {
        if (changes.editAllow &&
            this.editAllow === false &&
            !changes.editAllow.isFirstChange) {
            this.closeClick();
        }
        if (Object.keys(changes).indexOf('inpRenderedText') > -1
            && typeof (this.inpRenderedText) === 'undefined') {
            console.warn('Markdown component change :: renderedText is passed undefined');
            this.renderedText = '';
        }
        else {
            this.renderedText = this.inpRenderedText;
        }
        if (Object.keys(changes).indexOf('inpRawText') > -1
            && typeof (this.inpRawText) === 'undefined') {
            console.warn('Markdown component change :: rawText is passed undefined');
            this.rawText = '';
        }
        else {
            this.rawText = this.inpRawText;
        }
    };
    MarkdownComponent.prototype.ngOnInit = function () {
        if (typeof (this.renderedText) === 'undefined') {
            console.warn('Markdown component init :: renderedText is passed undefined');
            this.renderedText = '';
        }
        if (typeof (this.rawText) === 'undefined') {
            console.warn('Markdown component init :: rawText is passed undefined');
            this.rawText = '';
        }
    };
    MarkdownComponent.prototype.ngAfterViewChecked = function () {
        if (this.editorBox) {
            this.boxHeight = this.editorBox.nativeElement.offsetHeight;
            if (this.boxHeight > this.renderedHeight) {
                this.enableShowMore = true;
            }
            else {
                this.enableShowMore = false;
            }
            this.cdr.detectChanges();
        }
        this.checkInputsDisabled();
    };
    MarkdownComponent.prototype.onInputEvent = function (event) {
        console.log('In-Markup Markdown input Event detected for input type: ' + event.type +
            ' with extraData ' + JSON.stringify(event.extraData));
        // we only support this interaction on checkboxes for now.
        // the mechanic is generic, add other controls below. In case,
        // you need to also add support for them in github-link-area as well.
        if (event.type === 'checkbox') {
            // disable the inputs so we don't get into async issues when
            // storing to the upstream component. This has to be done on
            // new supported input types as well.
            this.inputsDisabled = true;
            // process the checkbox clicked, find the markdown markup reference, update it.
            var activatedCheckboxIndex = event.extraData.checkboxIndex;
            var checked = event.extraData.checked;
            var markdownMarkup = this.rawText;
            // the JavaScript RegExp flavour makes it hard to to nth occurence
            // expressions, so we're doing it by hand here.
            var regex = /^ *[-*] *\[[ xX]*\]/gm;
            var m = void 0;
            var matchIndex = 0;
            // tslint:disable-next-line:no-conditional-assignment
            while ((m = regex.exec(markdownMarkup)) !== null) {
                // This is necessary to avoid infinite loops with zero-width matches.
                if (m.index === regex.lastIndex) {
                    regex.lastIndex++;
                }
                if (matchIndex === activatedCheckboxIndex && m.length > 0) {
                    // JavaScript does not have a replace by index method.
                    var matchLen = m[0].length;
                    var matchEndIndex = regex.lastIndex;
                    var matchStartIndex = matchEndIndex - matchLen;
                    var replaceStr = void 0;
                    if (checked) {
                        replaceStr = m[0].replace(/\[[ ]*\]/, '[x]');
                    }
                    else {
                        replaceStr = m[0].replace(/\[[xX]+\]/, '[ ]');
                    }
                    markdownMarkup =
                        markdownMarkup.substring(0, matchStartIndex) +
                            replaceStr +
                            markdownMarkup.substring(matchEndIndex, markdownMarkup.length);
                }
                matchIndex++;
            }
            this.rawText = markdownMarkup;
            // signal that the markdown has changed to the outside world.
            this.saveClick();
        }
        else {
            console.log('Input type ' + event.type + ' is not supported yet.');
        }
    };
    MarkdownComponent.prototype.onClickMarkdownTab = function () {
        if (this.viewType === 'preview') {
            this.viewType = 'markdown';
        }
    };
    MarkdownComponent.prototype.onClickPreviewTab = function () {
        var _this = this;
        if (this.viewType === 'markdown') {
            if (this.rawText !== this.editorInput.nativeElement.innerText.trim()) {
                // Emit raw text to get preview
                this.rawText = this.editorInput.nativeElement.innerText.trim();
                this.showPreview.emit({
                    rawText: this.rawText,
                    callBack: function (t, m) { return _this.renderPreview(t, m); }
                });
                this.rendering = true;
            }
            else {
                this.viewType = 'preview';
            }
        }
    };
    MarkdownComponent.prototype.checkInputsDisabled = function () {
        this.setPreviewInputsDisabled('input.markdown-checkbox', this.inputsDisabled);
    };
    // disables/enables the inputs on the rendered markup
    MarkdownComponent.prototype.setPreviewInputsDisabled = function (query, disabled) {
        var el = this.previewArea;
        if (el) {
            var queryElems = el.nativeElement.querySelectorAll(query);
            if (queryElems && queryElems.length > 0) {
                // we need to use a classic loop instead of forEach here
                // as forEach on NodeLists is not supported on every browser.
                // Example: Chrome works, but Protractor not.
                for (var i = 0; i < queryElems.length; ++i) {
                    if (disabled) {
                        queryElems[i].setAttribute('disabled', disabled ? 'true' : 'false');
                    }
                    else {
                        queryElems[i].removeAttribute('disabled');
                    }
                }
            }
        }
    };
    MarkdownComponent.prototype.enableEditor = function (event) {
        if (event) {
            event.stopPropagation();
        }
        if (this.rawText === '') {
            this.activeEditor();
        }
        this.inputsDisabled = true;
    };
    MarkdownComponent.prototype.activeEditor = function () {
        if (this.editAllow) {
            // Activate the editor
            this.editorActive = true;
            // Show the markdown default view
            this.viewType = 'markdown';
            // Save current values
            this.previousRawText = this.rawText;
            this.previousRenderedText = this.renderedText;
            this.onActiveEditor.emit(true);
        }
    };
    MarkdownComponent.prototype.deactivateEditor = function () {
        // Deactivate the editor
        this.editorActive = false;
        // deactivate inputs when in edit mode
        this.inputsDisabled = false;
        // Show the preview default view
        this.viewType = 'preview';
    };
    MarkdownComponent.prototype.saveClick = function () {
        var _this = this;
        if (this.allowEmptySave || (!this.fieldEmpty && !this.allowEmptySave)) {
            if (this.viewType === 'markdown' &&
                this.previousRawText !== this.editorInput.nativeElement.innerText.trim()) {
                this.saving = true;
                this.onSaveClick.emit({
                    rawText: this.editorInput.nativeElement.innerText.trim(),
                    callBack: function (t, m) { return _this.saveUpdate(t, m); }
                });
            }
            else if (this.viewType === 'preview' &&
                this.previousRawText !== this.rawText) {
                this.saving = true;
                this.onSaveClick.emit({
                    rawText: this.rawText,
                    callBack: function (t, m) { return _this.saveUpdate(t, m); }
                });
            }
            else {
                this.deactivateEditor();
            }
        }
    };
    MarkdownComponent.prototype.editorKeyUp = function (event) {
        // Do not use this.event.srcElement. The "event" object is not consistent
        // across browsers. Chrome has event.srcElement while firefox has event.originalTarget.
        this.fieldEmpty = this.editorInput.nativeElement.innerText.trim() === '';
    };
    MarkdownComponent.prototype.closeClick = function () {
        // Restore saved previous data
        this.rawText = this.previousRawText;
        this.renderedText = this.previousRenderedText;
        this.deactivateEditor();
        this.onCloseClick.emit();
    };
    MarkdownComponent.prototype.renderPreview = function (rawText, renderedText) {
        this.rendering = false;
        if (typeof (rawText) === 'undefined') {
            console.warn('Markdown component preview callback :: rawText is passed undefined');
            this.rawText = '';
        }
        else {
            this.rawText = rawText;
        }
        if (typeof (renderedText) === 'undefined') {
            console.warn('Markdown component preview callback :: renderedText is passed undefined');
            this.renderedText = '';
        }
        else {
            this.renderedText = renderedText;
        }
        this.viewType = 'preview';
    };
    MarkdownComponent.prototype.saveUpdate = function (rawText, renderedText) {
        this.saving = false;
        if (typeof (rawText) === 'undefined') {
            console.warn('Markdown component save callback :: rawText is passed undefined');
            this.rawText = '';
        }
        else {
            this.rawText = rawText;
        }
        if (typeof (renderedText) === 'undefined') {
            console.warn('Markdown component save callback :: renderedText is passed undefined');
            this.renderedText = '';
        }
        else {
            this.renderedText = renderedText;
        }
        // finally, deactivate the editor.
        this.deactivateEditor();
    };
    MarkdownComponent.prototype.lengthToggle = function (showMore) {
        this.showMore = showMore;
    };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], MarkdownComponent.prototype, "fieldName", void 0);
    __decorate([
        Input('renderedText'),
        __metadata("design:type", Object)
    ], MarkdownComponent.prototype, "inpRenderedText", void 0);
    __decorate([
        Input('rawText'),
        __metadata("design:type", String)
    ], MarkdownComponent.prototype, "inpRawText", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], MarkdownComponent.prototype, "rendering", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], MarkdownComponent.prototype, "saving", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], MarkdownComponent.prototype, "placeholder", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], MarkdownComponent.prototype, "editAllow", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], MarkdownComponent.prototype, "renderedHeight", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], MarkdownComponent.prototype, "allowEmptySave", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], MarkdownComponent.prototype, "onActiveEditor", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], MarkdownComponent.prototype, "onSaveClick", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], MarkdownComponent.prototype, "showPreview", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], MarkdownComponent.prototype, "onCloseClick", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], MarkdownComponent.prototype, "onClickOut", void 0);
    __decorate([
        HostListener('document:click', ['$event.target']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [HTMLElement]),
        __metadata("design:returntype", void 0)
    ], MarkdownComponent.prototype, "onClick", null);
    __decorate([
        ViewChild('editorInput'),
        __metadata("design:type", ElementRef)
    ], MarkdownComponent.prototype, "editorInput", void 0);
    __decorate([
        ViewChild('editorBox'),
        __metadata("design:type", ElementRef)
    ], MarkdownComponent.prototype, "editorBox", void 0);
    __decorate([
        ViewChild('previewArea'),
        __metadata("design:type", ElementRef)
    ], MarkdownComponent.prototype, "previewArea", void 0);
    MarkdownComponent = __decorate([
        Component({
            encapsulation: ViewEncapsulation.None,
            selector: 'f8-markdown',
            styles: [require('./markdown.component.css').toString()],
            template: "<ul class=\"nav nav-tabs\" *ngIf=\"editorActive\"><li [class.active]=\"viewType==='markdown'\"><a (click)=\"onClickMarkdownTab()\" class=\"tab-btn\">Markdown</a></li><li [class.active]=\"viewType==='preview'\"><a (click)=\"onClickPreviewTab()\" class=\"tab-btn\">{{rendering ? \"Loading..\" : \"Preview\"}}</a></li></ul><div class=\"editor-container\"><div class=\"editor\" #previewArea (click)=\"enableEditor($event);\" [class.active]=\"editorActive\" [class.show-less]=\"!showMore && !editorActive\"><div class=\"editor-icon-container\"><i *ngIf=\"editAllow\" (click)=\"activeEditor()\" class=\"pficon-edit edit-icon\"></i></div><p almEditable #editorInput *ngIf=\"viewType==='markdown'\" [editable]=\"true\" class=\"editor-box editor-markdown\" (keyup)=\"editorKeyUp($event)\" [innerText]=\"rawText\"></p><div #editorBox #previewArea *ngIf=\"viewType==='preview' && ( renderedText.length || ( renderedText.changingThisBreaksApplicationSecurity && renderedText.changingThisBreaksApplicationSecurity.length ) )\" class=\"editor-box editor-preview markdown-rendered\"><github-link-area [content]=\"renderedText\" (onInputEvent)=\"onInputEvent($event)\"></github-link-area></div><div #editorBox *ngIf=\"viewType==='preview' && !renderedText.length && (!renderedText.changingThisBreaksApplicationSecurity || !renderedText.changingThisBreaksApplicationSecurity.length)\" class=\"editor-box editor-preview placeholder\">{{placeholder}}</div></div></div><div class=\"actions\" *ngIf=\"!editorActive && enableShowMore\"><small class=\"pull-right clickable\"><a *ngIf=\"!showMore\" (click)=\"lengthToggle(!showMore)\">Show more</a> <a *ngIf=\"showMore\" (click)=\"lengthToggle(!showMore)\">Show less</a></small></div><div class=\"actions\" *ngIf=\"editorActive\"><small><b>Markdown Supported</b> </small><button (click)=\"closeClick()\" class=\"fl btn btn-default pull-right action-btn\"><i class=\"fa fa-close\"></i></button> <button (click)=\"saveClick()\" class=\"fl btn btn-primary pull-right action-btn btn-save\" [disabled]=\"saving || (!allowEmptySave && fieldEmpty)\"><i class=\"fa fa-check\"></i></button></div>"
        })
        /**
         * Implements a markdown component. Important: in order that the checkbox
         * feature is working, the rendered content set using the attribute needs
         * to be wrapped into a SafeValue instance. This prevents the sanitizer
         * to strip the input fields needed for the checkboxes from the rendered
         * html.
         */
        ,
        __metadata("design:paramtypes", [ChangeDetectorRef, ElementRef])
    ], MarkdownComponent);
    return MarkdownComponent;
}());
export { MarkdownComponent };
//# sourceMappingURL=markdown.component.js.map